#! /usr/bin/env python3

import subprocess, os, sys, math, argparse

try:
    import matplotlib, numpy
    import matplotlib.pyplot as plt
    useMatplolib = True
except:
    useMatplolib = False

parser = argparse.ArgumentParser()
parser.add_argument('-n', '--samples', type=int, default=10)
parser.add_argument('-m', '--length', type=int, default=10)
parser.add_argument('-d', '--distance', type=int, default=1)
parser.add_argument('--variation', action="store_true")
args = parser.parse_args()

m = args.length
n = args.samples
d = args.distance
md = int(m/d)

m1 = [0.0 for i in range(md)]
m2 = [0.0 for i in range(md)]

for i in range(args.samples):

    subprocess.call(['./build/ex_1_1', str(m+2), str(args.distance)])
    with open('results.dat') as f:
        for l in f:
            ll = l.split()
            it = int(ll[0]) - 1
            t = float(ll[1])
            if i == 0:
                m1[it] = t
                m2[it] = 0.0
            else:
                m1[it] += t
                e = (i+1)*t - m1[it]
                m2[it] += (e*e)/(i*(i+1))

for i in range(md):
    m1[i] /= n
    m2[i] = math.sqrt(m2[i]/n)

if useMatplolib:
    plt.figure()
    x = numpy.linspace(1,md, num=md)
    y = numpy.array(m1)
    dy = numpy.array(m2)
    p1 = plt.plot(x, y, "b-o", label="moyenne sur " + str(n) + ' exécutions')
    if args.variation:
        p2 = plt.errorbar(x, y, yerr=dy, ecolor='r', capsize=4)
    plt.xlabel('Iteration')
    plt.legend()
    plt.title('Temps Iteration n')
    plt.savefig('cycles.pdf') 
    plt.show()
else:
    print('Temps Iteration n')
    for i in range(len(m1)):
        print("{:>5d}  {:12.3e}".format(i+1, m1[i]))

