#include "timer.hxx"
#include <stdlib.h>
#include <chrono>
#include <iostream>

#ifdef USE_PAPI
#include <papi.h>

int EventSet;

void Timer::init()
{
  int retval = PAPI_library_init(PAPI_VER_CURRENT);
  
  if (retval != PAPI_VER_CURRENT && retval > 0) {
    fprintf(stderr, "PAPI library version mismatch!\n");
    exit(-1); 
  }

  if (retval < 0) {
    fprintf(stderr, "PAPI initialisation error\n");
    exit(-1); 
  }

  EventSet = PAPI_NULL;
  if (PAPI_create_eventset(&EventSet) != PAPI_OK) {
    fprintf(stderr, "PAPI create event set error\n");
    exit(-1); 
  }

  if (PAPI_add_event(EventSet, PAPI_TOT_CYC) != PAPI_OK) {
    fprintf(stderr, "PAPI add event error\n");
    exit(-1); 
  }

}

void Timer::start()
{
  if (PAPI_start(EventSet) != PAPI_OK) {
    std::cerr << name << " PAPI start timer error\n";
    exit (-1);
  }
}

void Timer::stop() 
{
  long long d;
  if (PAPI_stop(EventSet, &d) != PAPI_OK) {
    std::cerr << name << " PAPI stop timer error\n";
    exit (-1);
  }
  _elapsed += d;
}

#else

struct TimerImpl {
  std::chrono::time_point<std::chrono::high_resolution_clock> start;
  std::chrono::duration<double> diff;
};

void Timer::init()
{
}

void Timer::start()
{
  _impl->start = std::chrono::high_resolution_clock::now();
}

void Timer::stop() 
{
  auto end = std::chrono::high_resolution_clock::now();
 
  _impl->diff = end - _impl->start;
  _elapsed += _impl->diff.count();
}

#endif

Timer::Timer(const char * name) : _name(name), _elapsed(0.0)
{
  _impl = new TimerImpl();
}

Timer::~Timer()
{
  delete _impl;
}